@@ -7,6 +7,7 @@ const System = imports.system;
 imports.gi.versions.Gtk = '3.0';
 
 const Gio = imports.gi.Gio;
+const GjsTestTools = imports.gi.GjsTestTools;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
@@ -352,6 +353,27 @@ describe('GObject class with decorator', function () {
         expect(notifySpy).toHaveBeenCalledTimes(2);
     });
 
+    it('disconnects connect_object signals on destruction', function () {
+        let callback = jasmine.createSpy('callback');
+        callback.myInstance = myInstance;
+        const instance2 = new MyObject();
+        instance2.connect_object('empty', callback, myInstance, 0);
+
+        instance2.emitEmpty();
+        instance2.emitEmpty();
+
+        expect(callback).toHaveBeenCalledTimes(2);
+
+        GjsTestTools.save_weak(myInstance);
+        myInstance = null;
+        callback = null;
+
+        System.gc();
+        System.gc();
+
+        expect(GjsTestTools.get_weak()).toBeNull();
+    });
+
     it('can define its own signals', function () {
         let emptySpy = jasmine.createSpy('emptySpy');
         myInstance.connect('empty', emptySpy);
@@ -910,14 +932,14 @@ describe('GObject virtual function', function () {
         })).toThrow();
     });
 
-    it('are defined also for static virtual functions', () => {
-        const CustomEmptyGICon = GObject.registerClass({
+    it('are defined also for static virtual functions', function () {
+        const CustomEmptyGIcon = GObject.registerClass({
             Implements: [Gio.Icon],
-        }, class CustomEmptyGICon extends GObject.Object {});
+        }, class CustomEmptyGIcon extends GObject.Object {});
         expect(Gio.Icon.deserialize).toBeInstanceOf(Function);
-        expect(CustomEmptyGICon.deserialize).toBe(Gio.Icon.deserialize);
+        expect(CustomEmptyGIcon.deserialize).toBe(Gio.Icon.deserialize);
         expect(Gio.Icon.new_for_string).toBeInstanceOf(Function);
-        expect(CustomEmptyGICon.new_for_string).toBe(Gio.Icon.new_for_string);
+        expect(CustomEmptyGIcon.new_for_string).toBe(Gio.Icon.new_for_string);
     });
 });
 
@@ -1176,6 +1198,8 @@ describe('Property bindings', function () {
     });
 
     it('can be set up as a group', function () {
+        if (GObject.BindingGroup === undefined)
+            pending('GLib version too old');
         const group = new GObject.BindingGroup({source: a});
         group.bind('string', b, 'string', GObject.BindingFlags.NONE);
         a.string = 'foo';
@@ -1184,6 +1208,8 @@ describe('Property bindings', function () {
     });
 
     it('can be set up as a group with custom mappings', function () {
+        if (GObject.BindingGroup === undefined)
+            pending('GLib version too old');
         const group = new GObject.BindingGroup({source: a});
         group.bind_full('bool', b, 'string', GObject.BindingFlags.NONE,
             (bind, source) => [true, `${source}`],
